import React, { useEffect, useState } from 'react'
import { Table, Input, Button, Form, Radio, DatePicker, Pagination } from 'antd'
import { IDvaState, namespace } from '@/models/order-manage/purchase/list'
import { connect } from 'react-redux'
import { IDispatch } from '@/models/connect'
import styles from './style.module.scss'
import { DownOutlined, UpOutlined } from '@ant-design/icons'
import classnames from 'classnames'
import columns from './columns'
import { ListLayout, Content } from '@/pages/layout/list-layout'
import orderCancel from '../components/cancel-order-modal'
import moment, { Moment } from 'moment'
import { RadioChangeEvent } from 'antd/lib/radio'
import { PURCHASE_ORDER_DETAIL } from '@/router/config/order-manage/path'
import { Link } from 'react-router-dom'
import { usePagination } from '@/pages/hooks/usePagination'
import { searchDefaultValue } from '@/pages/hooks/useSearch'
import { PaginationWrapper, pageSizeOptions, showTotal, defaultPageOptions } from '@/components/PaginationWrapper'
import { useDebounceFn } from '@umijs/hooks'

const { Item: FormItem } = Form
const { RangePicker } = DatePicker
const dateFormat = 'YYYY-MM-DD'
const endDate: Moment = moment()
const startDate: Moment = moment().add(-3, 'month')
const dateTimeFormat = dateFormat + ' HH:mm:ss'
const Search = Input.Search

interface IProps extends IDvaState, IDispatch {}

type Order = defs.OrderPageInfoDetailReponse

interface GoodsInfoCellProps {
  record: Order
  dataIndex: string
  editable: boolean
  children: any
}

const OrderInfoCell: React.FC<GoodsInfoCellProps> = ({ record, dataIndex, editable, children, ...restProps }) => {
  const [openStatus, setOpenStatus] = useState(false)
  function toggleOpenStatus() {
    const currentOpenStatus = !openStatus
    setOpenStatus(currentOpenStatus)
  }
  if (!editable) {
    return <td {...restProps}>{children}</td>
  }
  // bugfix: data在某种极端情况下为undefined
  let data = []
  if (record && record.orderGoodsInfoResponseList) {
    data = [...record.orderGoodsInfoResponseList]
  }
  return (
    <td {...restProps}>
      <div className={classnames(styles.orderInfoContainer, { [styles.orderInfoContainerClose]: !openStatus })}>
        <div className={styles.itemList}>
          {data.map((item, id) => (
            <p key={id} className={styles.item}>
              {item.goodsName}
            </p>
          ))}
        </div>
        <div className={styles.countList}>
          {data.map((item, id) => (
            <p key={id} className={styles.count}>
              X{item.number}
            </p>
          ))}
        </div>
      </div>
      {data.length > 3 ? (
        <p onClick={toggleOpenStatus} className={styles.moreBtn}>
          <span style={{ marginRight: 4 }}>{openStatus ? '收起' : '展开'}</span>
          {openStatus ? <UpOutlined /> : <DownOutlined />}
        </p>
      ) : null}
    </td>
  )
}
const initDate: Moment[] = [startDate, endDate]
const initStatus = 'all'
const OrderList: React.FC<IProps> = ({ orderList, dispatch, totalCount }) => {
  const [status, setStatus] = useState(initStatus)
  const [rangeDateVaule, setRangeDateVaule] = useState<any>(initDate)
  const [pageOptions, setPagination] = usePagination(defaultPageOptions)
  const [keywords, handleInputChange] = useState()
  const { run: query } = useDebounceFn(() => {
    const [start, end] = rangeDateVaule
    dispatch({
      type: `${namespace}/queryOrderList`,
      payload: {
        ...pageOptions,
        startTime: start.startOf('day').format(dateTimeFormat),
        endTime: end.endOf('day').format(dateTimeFormat),
        status,
        keywords
      }
    })
    return () => dispatch({ type: `${namespace}/clearData` })
  }, 300)
  useEffect(query, [pageOptions.page, pageOptions.size, rangeDateVaule, status, keywords])
  function handlePageSizeChange(page, size) {
    setPagination({
      page,
      size
    })
  }
  /**
   * 重置页码和关键字
   */
  function resetKeyWordsAndPage() {
    handleInputChange(searchDefaultValue)
    setPagination({
      page: 1
    })
  }
  /**
   * radio change 事件
   * @param e radio值
   */
  function handleTypeChange(e: RadioChangeEvent) {
    const { value } = e.target
    resetKeyWordsAndPage()
    setStatus(value)
  }
  /**
   * 日期选择事件
   * @param dates Date日期
   */
  function handleDateChange(dates: Moment[]) {
    resetKeyWordsAndPage()
    setRangeDateVaule(dates)
  }
  function handleSearchInputChange(e) {
    setPagination({
      page: 1,
      size: pageOptions.size
    })
    setRangeDateVaule(initDate)
    setStatus(initStatus)
    handleInputChange(e.target.value)
  }
  const components = {
    body: {
      cell: OrderInfoCell
    }
  }
  const COLUMNS: any = columns.map(col => {
    if (col.editable) {
      return {
        ...col,
        onCell: record => ({
          record,
          editable: col.editable,
          dataIndex: col.dataIndex
        })
      }
    }
    if (col.outsideRender) {
      return {
        ...col,
        render: (_, record: Order) => {
          const cancelJsx = (
            <p style={{ textAlign: 'center', paddingTop: '10px' }}>
              <Button onClick={() => orderCancel(dispatch, record.id, () => query())}>取消订单</Button>
            </p>
          )

          /**
           * created 待接单/accepted 待发货/accepted_part 待发货(显示商品收货)
           * sending,produced,produced_part 待收货(显示商品收货)
           * receive,finished 已完成 / canceled 已取消
           */
          if (record.status === 'created') {
            return cancelJsx
          }
          if (
            // record.isReceipt
            // 除了以下状态，都显示商品收货
            record.status !== 'receive' &&
            record.status !== 'finished' &&
            record.status !== 'canceled' &&
            record.status !== 'accepted'
          ) {
            return (
              <>
                <p style={{ textAlign: 'center' }}>
                  <Link to={{ pathname: PURCHASE_ORDER_DETAIL, search: `?orderId=${record.id}&tab=2` }}>
                    <Button type="primary">商品收货</Button>
                  </Link>
                </p>
              </>
            )
          }
        }
      }
    }
    return col
  })
  return (
    <ListLayout>
      <Form>
        <FormItem label="订单状态" className={styles.breakLine}>
          <Radio.Group value={status} onChange={handleTypeChange}>
            <Radio value="all">全部</Radio>
            <Radio value="created">待接单</Radio>
            <Radio value="accepted">待发货</Radio>
            <Radio value="produced">待收货</Radio>
            <Radio value="finished">已完成</Radio>
            <Radio value="canceled">已取消</Radio>
          </Radio.Group>
        </FormItem>
        <FormItem label="下单时间">
          <RangePicker allowClear={false} format={dateFormat} defaultValue={[startDate, endDate]} onChange={handleDateChange} />
        </FormItem>
      </Form>
      <Form layout="inline">
        <FormItem>
          <Search
            style={{ width: 400 }}
            placeholder="订单编号/供应商"
            enterButton="查询"
            size="large"
            onSearch={query}
            onChange={handleSearchInputChange}
            value={keywords}
          />
        </FormItem>
        {/* <FormItem>
          <Button type="primary" size="large">
            导出
          </Button>
        </FormItem> */}
      </Form>
      <Content>
        <Table
          rowKey="id"
          columns={COLUMNS as any}
          components={components as any}
          dataSource={orderList}
          className={styles.orderListTable}
          rowClassName={() => styles.row}
          tableLayout="fixed"
          pagination={false}
        />
        <PaginationWrapper>
          <Pagination
            pageSize={pageOptions.size}
            pageSizeOptions={pageSizeOptions}
            current={pageOptions.page}
            onChange={handlePageSizeChange}
            total={totalCount}
            showSizeChanger
            showQuickJumper
            onShowSizeChange={handlePageSizeChange}
            showTotal={showTotal}
          />
        </PaginationWrapper>
      </Content>
    </ListLayout>
  )
}

const mapStateToProps = models => ({
  orderList: models[namespace].orderList,
  totalCount: models[namespace].totalCount
})

export default connect(mapStateToProps)(OrderList)
