import React, { useMemo, useState, useEffect } from 'react'
import { Input, Button, Popover } from 'antd'
import styles from './style.module.scss'
import { namespace, IPurchaseHome, MenuItem, ALL_ID } from '@/models/purchase-module/purchase-home'
import { connect } from 'react-redux'
import { IDispatch } from '@/models/connect'
import { DownOutlined } from '@ant-design/icons'
import defaultImg from './default.png'

const isValidId = (id: string) => {
  return id && id !== ALL_ID
}

interface IProps extends IDispatch, Partial<IPurchaseHome> {}

function getLabelAndSubMenu(menus: MenuItem[], selectedId: number | string): [string, MenuItem[]] {
  if (selectedId === ALL_ID) {
    return ['全部商品', []]
  }
  for (const item of menus) {
    // tips: 当前数据有问题
    if (selectedId === item.id) {
      return [item.categoryName, item.children]
    }
  }
  return ['全部商品', []]
}

const TopSearch: React.FC<IProps> = ({ dispatch, menus, selectedId, id, keywords }) => {
  const [selectedSubMenu, setSelectedSubMenu] = useState(ALL_ID)
  // selected id变化后，重置selectedSubmenuId
  useEffect(() => {
    setSelectedSubMenu(ALL_ID)
  }, [selectedId])
  const [visible, setVisible] = useState(false)
  function queryGoodsList(id: number | string, keywords: string) {
    // 请求数据
    dispatch({
      type: `${namespace}/queryGoodsList`,
      payload: {
        page: 1,
        size: 30,
        categoryId: id,
        goodsName: keywords
      }
    })
  }
  function handleInputChange(e: React.ChangeEvent<HTMLInputElement>) {
    const keywords = e.target.value
    if (keywords === '') {
      queryGoodsList(id, keywords)
    }
    dispatch({
      type: `${namespace}/setState`,
      payload: {
        keywords
      }
    })
  }
  function handleSearch(value) {
    // 重置页码
    dispatch({
      type: `${namespace}/setState`,
      payload: {
        page: 1
      }
    })
    // 搜索
    queryGoodsList(id, value)
  }
  function handleClickSubMenu(id: string) {
    setVisible(false)
    const fid = id === ALL_ID ? selectedId : id
    // 保存key
    dispatch({
      type: `${namespace}/setState`,
      payload: {
        id: fid,
        page: 1,
        keywords: ''
      }
    })
    setSelectedSubMenu(id)
    queryGoodsList(fid, '')
  }
  function handleVisibleChange(visible) {
    setVisible(visible)
  }
  function handleExitQuery() {
    if (keywords === '') {
      return
    }
    // 保存关键字
    dispatch({
      type: `${namespace}/setState`,
      payload: {
        keywords: '',
        page: 1
      }
    })
    queryGoodsList(id, '')
  }
  const [categoryName, subMenus] = useMemo(() => getLabelAndSubMenu(menus, selectedId), [menus, selectedId])
  function genSubMenu(subMenus: MenuItem[]) {
    return (
      <div className={styles.submenuContentWrapper}>
        <div className={styles.item} onClick={() => handleClickSubMenu(ALL_ID)}>
          <img src={defaultImg} alt="分类图片" />
          <p className={styles.twoRowsTitle}>全部</p>
        </div>
        {subMenus.map(item => (
          <div key={item.id} className={styles.item} onClick={() => handleClickSubMenu(item.id)}>
            <img src={item.bgImageUrl || defaultImg} alt="分类图片" />
            <p className={styles.twoRowsTitle}>{item.categoryName}</p>
          </div>
        ))}
      </div>
    )
  }
  function findSubMenuTitle(subMenus: MenuItem[], id: string) {
    if (id === ALL_ID) {
      return '全部'
    }
    for (const item of subMenus) {
      if (item.id === id) {
        return item.categoryName
      }
    }
    return '全部'
  }
  const subMenuTitle = findSubMenuTitle(subMenus, selectedSubMenu)
  return (
    <div className={styles.topSearchContainer}>
      <p className={styles.categoryLabel}>
        <span>
          {categoryName}
          {isValidId(selectedId) ? '/' : ''}
        </span>
        {isValidId(selectedId) && (
          <Popover onVisibleChange={handleVisibleChange} visible={visible} placement="bottomLeft" content={genSubMenu(subMenus)}>
            <b>
              {subMenuTitle}
              <DownOutlined className={styles.ico} />
            </b>
          </Popover>
        )}
      </p>
      <div className={styles.searchBar}>
        <Input.Search
          value={keywords}
          placeholder="输入商品名称查询"
          enterButton="查询"
          onChange={handleInputChange}
          onSearch={handleSearch}
        />
        <Button onClick={handleExitQuery}>退出查询</Button>
      </div>
    </div>
  )
}

const mapStateToProps = models => ({
  menus: models[namespace].menus,
  selectedId: models[namespace].selectedId,
  id: models[namespace].id,
  keywords: models[namespace].keywords
})

export default connect(mapStateToProps)(TopSearch)
