import config from '@/config'
import { parse } from 'qs'
import { getLocalStorage } from '../storage'

/**
 * 判断登陆状态，根据以下情况返回。false代表需要重新登陆，true则不需要
 *- 如果没有token，返回false
 *- 如果token不是后端返回的规则，解析失败，返回false
 *- 如果后端token存在，且解析成功，如果剩余时间小于配置时间返回false
 *- 如果后端token存在，且解析成功，如果剩余时间大于配置时间才返回true
 */
export function judgeLoginStatus(): boolean {
  try {
    const token: string = localStorage.getItem(config.authKey)
    if (token) {
      // 服务器返回的token规则不一致，所以暂时无法校验
      return true
    }
    return false
  } catch (e) {
    return false
  }
}

/**
 * 获取url参数
 */
export function getPageQuery() {
  return parse(window.location.href.split('?')[1])
}

/**
 *  手机号验证正则
 */
export const regMobile = /^1\d{10}/

/**
 * 导出 流方式
 *
 * const sendData = {
 *    Authorization: this.state.Authorization,
 *  }
 *  exportFile('/api/admin/platform/downloadReceiptProfitTemplate.xls', sendData)
 */
export const exportFile = (url, params, removeTime = 4000) => {
  const urlParams = []
  for (const p of Object.keys(params)) {
    const key = p
    const value = params[p]
    if (value !== undefined && value !== null && value !== '') {
      urlParams.push({
        key,
        value
      })
    }
  }
  urlParams.push({ key: 'Authorization', value: getLocalStorage(config.authKey) })
  const exportForm = document.createElement('form')
  exportForm.method = 'get'
  exportForm.action = url
  exportForm.style.display = 'none'
  urlParams.forEach(v => {
    const input = document.createElement('input')
    input.type = 'text'
    input.name = v.key
    input.value = v.value
    exportForm.appendChild(input)
  })

  document.body.appendChild(exportForm)

  exportForm.submit()

  setTimeout(() => {
    // 移除dom，避免越来越多
    document.body.removeChild(exportForm)
  }, removeTime)
}
/**
 * base64转文件
 * @param source base64编码的字符串
 * @param filename 生成的文件名
 */
export const convertBase64ToFile = (source, filename) => {
  function dataURLtoBlob(dataurl) {
    const arr = dataurl.split(',')

    const mime = arr.length === 1 ? 'octet-stream' : arr[0].match(/:(.*?);/)[1]
    const bstr = window.atob(arr[arr.length - 1])

    let n = bstr.length
    const u8arr = new Uint8Array(n)
    while (n--) {
      u8arr[n] = bstr.charCodeAt(n)
    }

    return new Blob([u8arr], { type: mime })
  }
  const blob = dataURLtoBlob(source)
  const dataUrl = window.URL.createObjectURL(blob)
  const link = document.createElement('a')
  link.href = dataUrl
  link.download = filename
  link.click()
}

/**
 * 判断`value`是否是数字
 * Checks if `value` is classified as a `Number` primitive or object
 * **Note:** To exclude `Infinity`, `-Infinity`,and `NaN`
 *
 * @param {*} value The value to check
 * @returns {boolean} Returns `true` if `value` is a number, else `false`
 * @example
 *
 * isNumber(3)
 * // => true
 *
 * isNumber(new Number(3))
 * // => true
 *
 * isNumber(NaN)
 * // => true
 *
 * isNumber(undefined)
 * // => false
 *
 * isNumber('3')
 * // => false
 */

const numberTag = '[object Number]'

function isObjectLike(value) {
  return typeof value === 'object' && value !== null
}

export function isNumber(value) {
  return typeof value === 'number' || (isObjectLike(value) && Object.prototype.toString.call(value) === numberTag)
}

/**
 * 返回限制在`lower`和`upper`之间的值
 * Clamps `number` within the inclusive `lower` and `upper` bounds.
 *
 * @param {number} number The number to clamp
 * @param {number} [lower] The lower bound
 * @param {number} upper The upper bound
 * @returns {number} Returns the clamped number.
 * @example
 *
 * clamp(-10, -5, 5)
 * // => -5
 *
 * clamp(10, -5, 5)
 * // => 5
 *
 */
const symbolTag = '[object Symbol]'
const NAN = 0 / 0

function isSymbol(value) {
  return typeof value === 'symbol' || (isObjectLike(value) && Object.prototype.toString.call(value) === symbolTag)
}

function toNumber(value) {
  if (typeof value === 'number') {
    return value
  }

  if (isSymbol(value)) {
    return NAN
  }

  return Number(value)
}

function baseClamp(num, lower, upper) {
  if (!isNaN(num)) {
    if (upper !== undefined) {
      num = num < upper ? num : upper
    }
    if (lower !== undefined) {
      num = num > lower ? num : lower
    }
  }
  return num
}

export function clamp(num, lower, upper) {
  if (upper === undefined) {
    upper = lower
    lower = undefined
  }
  if (upper !== undefined) {
    upper = toNumber(upper)
    upper = !isNaN(upper) ? upper : 0
  }
  if (lower !== undefined) {
    lower = toNumber(lower)
    lower = !isNaN(lower) ? lower : 0
  }
  return baseClamp(toNumber(num), lower, upper)
}

/**
 * 将输入的价钱格式化
 * @param {number} number
 * 3082 => 3082
 * 34.334 => 34.33
 * 34.335 => 34.34
 * 33.4 => 33.40
 */
export function formatMoney(val: number | string) {
  const v = Number(val)
  // if (Number.isInteger(v)) {
  //   return v
  // }
  return v.toFixed(2)
  // return parseFloat(v.toFixed(2))
}
/**
 * 阿里云cdn图片宽高压缩
 * 如不能准确获取宽高，则传入一个相近的宽高，但是比例要相同，否则压缩比例与设计稿不同，会出现视觉异常
 * @param {string} 图片url
 * @param {number} 图片宽度
 * @param {number} 图片高度
 */
export function handleImageSizeUrl(url: string, width: number | string = 100, height: number | string = 100) {
  if (!url) {
    return null
  }
  const sizeLimit = `image/resize,m_mfit,w_${parseInt(width as string, 0)},h_${parseInt(
    height as string,
    0
  )},limit_0/auto-orient,1/quality,q_90/format,jpg/interlace,1`
  return `${url}?x-oss-process=${sizeLimit}`
}

/**
 * 获取参数
 * @param {string} name
 * @returns
 */
export function getQueryString(name: string) {
  const reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i')
  const r = window.location.search.substr(1).match(reg)
  if (r != null) {
    return unescape(decodeURI(r[2]))
  }
  return null
}

export function executeCallback(callbak) {
  // tslint:disable-next-line: no-unused-expression
  typeof callbak === 'function' && callbak()
}
